/*==============================================================================
 *
 *  The CHT_AutoFibo program is free software: you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CHT_AutoFibo is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 *  A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CHT_AutoFibo. If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  FILE: CHT_AutoFibo_0.1-zz.mq4
 *  VERSION: 0.1-b2
 *  AUTHOR: Thierry Chappuis <tc77@pygnol.ch>
 *
 *  DESCRIPTION:
 *  MT4 Custom indicator to automatically draw fibonacci lines suitable to trade
 *  ABCD patterns.
 *
 *  LOG:
 *  - 20121104: version 0.1
 *  - 20121207: version 0.1-zz, cycle detection using ZigZag
 *
 *  Copyright 2012, Thierry Chappuis
 *=============================================================================*/

#property copyright "Copyright 2012, Thierry Chappuis"
#property link      "tc77@pygnol.ch"

#property indicator_chart_window

#property indicator_buffers 5

#property indicator_color1 White
#property indicator_color2 Blue
#property indicator_color3 Red

/* Input parameters */
extern int AutoFibo_NumBars = 200;
extern color FiboColor = Yellow;
extern int ZigZag_ExtDepth = 12;
extern int ZigZag_ExtDeviation = 5;
extern int ZigZag_ExtBackstep = 3;
extern bool ZigZag_Visible = true;

double NB_FIBO_LEVELS = 18;
double FiboLevels[18];
double FiboRatios[18] =
{
    0.0,
    1.0,
    0.236,
    0.382,
    0.500,
    0.618,
    0.784,
    0.886,
    1.272,
    1.618,
    2.000,
    2.618,
    4.618,
    -0.272,
    -0.618,
    -1.000,
    -1.618,
    -3.618,
};
string FiboLabels[18] =
{
    "0.0",
    "100.0",
    "23.6",
    "38.2",
    "50.0",
    "61.8",
    "78.6",
    "88.6",
    "127.2",
    "161.8",
    "200.0",
    "261.8",
    "461.8",
    "-127.2",
    "-161.8",
    "-200.0",
    "-261.8",
    "-461.8",
};

/* Value Chart parameters */
int vc_NUM_BARS = 5;

/* Buffers */
double AutoFiboZigZag[];
double ZigZagHigh[];
double ZigZagLow[];
double vcHigh[];
double vcLow[];

/* Initialization function */
int init()
{
    SetIndexBuffer(0, AutoFiboZigZag);
    SetIndexBuffer(1, ZigZagHigh);
    SetIndexBuffer(2, ZigZagLow);
    SetIndexBuffer(3, vcHigh);
    SetIndexBuffer(4, vcLow);

    if (ZigZag_Visible == true)
    {
        SetIndexStyle(0, DRAW_SECTION);
        SetIndexStyle(1, DRAW_ARROW);
        SetIndexArrow(1, 159);
        SetIndexStyle(2, DRAW_ARROW);
        SetIndexArrow(1,159);
    }
    else
    {
        SetIndexStyle(0, DRAW_NONE);
        SetIndexStyle(1, DRAW_NONE);
        SetIndexStyle(2, DRAW_NONE);
    }
    SetIndexStyle(3, DRAW_NONE);
    SetIndexStyle(4, DRAW_NONE);

    SetIndexLabel(0, "AutoFiboZigZag");
    SetIndexLabel(1, "Price cycle high");
    SetIndexLabel(2, "Price cycle low");

    SetIndexEmptyValue(0, 0.0);
    SetIndexEmptyValue(1, 0.0);
    SetIndexEmptyValue(2, 0.0);
    SetIndexEmptyValue(3, 0.0);
    SetIndexEmptyValue(4, 0.0);

    Comment("CHT_AutoFibo_ZZ");
    return(0);
}

/* Entry point */
int start()
{
    int counted_bars = IndicatorCounted();

    value_chart(counted_bars);
    zigzag();
    detect_fibo_patterns();

    return(0);
}

int deinit()
{
    ObjectsDeleteAll(0);
    Comment("");
    return(0);
}

void value_chart(int counted_bars)
{
    int bars;

    // The last counted bar is counted again
    bars = Bars - counted_bars;

    if(counted_bars > 0)
    {
        bars = Bars - counted_bars + 1;
    }

    double sum;
    double floatingAxis;
    double volatilityUnit;

    for(int i = 0; i < bars; i++)
    {
        /* Determination of the floating axis */
        sum = 0;
        for (int k = i; k < vc_NUM_BARS + i; k++)
        {
            sum += (High[k] + Low[k]) / 2.0;
        }
        floatingAxis = sum / vc_NUM_BARS;

        /* Determination of the volatility unit */
        sum = 0;
        for (k = i; k < vc_NUM_BARS + i; k++)
        {
            sum += High[k] - Low[k];
        }
        volatilityUnit = 0.2 * (sum / vc_NUM_BARS);

        /* Determination of relative high, low, open and close values */
        vcHigh[i] = (High[i] - floatingAxis) / volatilityUnit; // relative high
        vcLow[i] = (Low[i] - floatingAxis) / volatilityUnit; // relative low
    }
    return(0);
}

int zigzag()
{
    int i;
    int back;
    double backval;
    double low, high;
    double lastlow, lasthigh;
    int lasthighpos,lastlowpos;

    for (i=Bars-ZigZag_ExtDepth; i>=0; i--)
    {
        low = Low[Lowest(NULL, 0, MODE_LOW, ZigZag_ExtDepth, i)];
        if (low == lastlow)
        {
            low = 0.0;
        }
        else
        {
            lastlow = low;
            if ((Low[i] - low) > (ZigZag_ExtDeviation*Point))
            {
                low = 0.0;
            }
            else
            {
                for (back = 1; back <= ZigZag_ExtBackstep; back++)
                {
                    backval = AutoFiboZigZag[i+back];
                    if ((backval != 0) && (backval > low))
                    {
                        AutoFiboZigZag[i+back] = 0.0;
                        ZigZagLow[i+back] = 0.0;
                    }
                }
            }
        }
        AutoFiboZigZag[i] = low;
        ZigZagLow[i] = low;
        
        high = High[Highest(NULL, 0, MODE_HIGH, ZigZag_ExtDepth, i)];
        if (high == lasthigh)
        {
            high=0.0;
        }
        else
        {
            lasthigh = high;
            if ((high - High[i]) > (ZigZag_ExtDeviation*Point))
            {
                high = 0.0;
            }
            else
            {
                for (back = 1; back <= ZigZag_ExtBackstep; back++)
                {
                    backval = ZigZagHigh[i+back];
                    if ((backval != 0) && (backval < high))
                    {
                        ZigZagHigh[i+back] = 0.0;
                    }
                }
            }
        }
        ZigZagHigh[i] = high;
    }

    lasthigh = -1;
    lasthighpos = -1;
    lastlow = -1;
    lastlowpos = -1;

    for (i = Bars-ZigZag_ExtDepth; i >= 0; i--)
    {
        low = AutoFiboZigZag[i];
        high = ZigZagHigh[i];
       
        if (high != 0)
        {
            if (lasthigh > 0)
            {
                if (lasthigh < high)
                {
                    ZigZagHigh[lasthighpos] = 0.0;
                }
                else
                {
                    ZigZagHigh[i] = 0.0;
                }
            }
            
            if (lasthigh < high || lasthigh < 0)
            {
                lasthigh = high;
                lasthighpos = i;
            }
            lastlow = -1;
        }

        if (low != 0)
        {
            if (lastlow > 0)
            {
                if(lastlow > low)
                {
                    AutoFiboZigZag[lastlowpos] = 0.0;
                    ZigZagLow[lastlowpos] = 0.0;
                }
                else
                {
                    AutoFiboZigZag[i] = 0.0;
                    ZigZagLow[i] = 0.0;
                }
            }

            if ((low < lastlow) || (lastlow < 0))
            {
                lastlow = low;
                lastlowpos = i;
            }
            lasthigh = -1;
        }
    }

    for(i = Bars-1; i >= 0; i--)
    {
        if (i >= Bars - ZigZag_ExtDepth)
        {
            AutoFiboZigZag[i] = 0.0;
        }
        else
        {
            backval = ZigZagHigh[i];
            if (backval != 0.0)
            {
                AutoFiboZigZag[i] = backval;
            }
        }
    }
}

void detect_fibo_patterns()
{
    int i, k;
    double zz[4];
    int zzpos[4];  
    double range;
    
    delete_fibo();

    for (i = AutoFibo_NumBars; i >= 0; i--)
    {
        if (AutoFiboZigZag[i] != 0.0)
        {
            for (k = 0; k < 3; k++)
            {
                zz[k] = zz[k+1];
                zzpos[k] = zzpos[k+1];
            }
            
            zz[k] = AutoFiboZigZag[i];
            zzpos[k] = i;
        }
    }
    
    range = zz[1] - zz[0];
    
    if (zz[0] < zz[1] && zz[0] <= zz[2] && zz[0] < zz[3])
    {
        if (zz[2] > zz[0] + 0.618 * range)
        {
            // Do nothing
        }
        
        if (zz[2] <= zz[0] + 0.618 * range && zz[2] > zz[0] + (1-0.786) * range)
        {
            // Draw fibo with target at 161.8
            draw_fibo(Time[zzpos[0]], zz[0], Time[zzpos[1]], zz[1], FiboColor, "161.8");
        }
        
        if (zz[2] <= zz[0] + (1-0.786) * range)
        {
            // Draw fibo with target at 127.2
            draw_fibo(Time[zzpos[0]], zz[0], Time[zzpos[1]], zz[1], FiboColor, "161.8");
        }
    }
    
    if (zz[0] > zz[1] && zz[0] >= zz[2] && zz[0] > zz[3])
    {
        if (zz[2] < zz[0] + 0.618 * range)
        {
            // Do nothing
        }
        
        if (zz[2] >= zz[0] + 0.618 * range && zz[2] < zz[0] + (1-0.786) * range)
        {
            // Draw fibo with target at 161.8
            draw_fibo(Time[zzpos[0]], zz[0], Time[zzpos[1]], zz[1], FiboColor, "161.8");
        }
        
        if (zz[2] >= zz[0] + (1-0.786) * range)
        {
            // Draw fibo with target at 127.2
            draw_fibo(Time[zzpos[0]], zz[0], Time[zzpos[1]], zz[1], FiboColor, "161.8");
        }
    }
}

void draw_fibo(datetime time1, double price1, datetime time2, double price2, color fibo_color, string target)
{
    int i;
    datetime start = time1;
    datetime end = Time[0];
    double range = price2 - price1;
    
    for (i = 0; i < NB_FIBO_LEVELS; i++)
    {
        draw_line(start, price1 + FiboRatios[i] * range, FiboLabels[i], fibo_color, FiboLabels[i] == target);
    }
}


void draw_line(datetime start, double level, string label, color col, bool is_target)
{
    string line_name = "Fibonacci line " + label;
    string label_name = "Fibonacci label " + label;
    
    delete_line(label);

    ObjectCreate(line_name, OBJ_TREND, 0, start, level, Time[0], level);
    ObjectSet(line_name, OBJPROP_STYLE, STYLE_DOT);
    ObjectSet(line_name, OBJPROP_COLOR, col);
    ObjectSet(line_name, OBJPROP_WIDTH, 1);
    ObjectSet(line_name, OBJPROP_RAY, false);
    
    ObjectCreate(label_name, OBJ_TEXT, 0, Time[0]+1500, level);
    ObjectSetText(label_name, label+" - "+DoubleToStr(level,Digits), 6, "Arial", EMPTY);
    ObjectSet(label_name, OBJPROP_COLOR, col);
        
    if (is_target == true)
    {
        ObjectSet(line_name, OBJPROP_STYLE, STYLE_SOLID);
        ObjectSet(line_name, OBJPROP_WIDTH, 2);
    }
}

void delete_line(string label)
{
    ObjectDelete("Fibonacci line " + label);
    ObjectDelete("Fibonacci label " + label);
}

void delete_fibo()
{
    for (int i = 0; i < NB_FIBO_LEVELS; i++)
    {
        delete_line(FiboLabels[i]);
    }
}